#!/usr/bin/env python
# from itertools import islice

import logging
import json
import datetime
import re

import rich.console
from itertools import islice
import pandas as pd
import yaml

import openpyxl
import os
import sys
import relecov_tools.utils
from relecov_tools.config_json import ConfigJson
import relecov_tools.json_schema

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class BioinfoMetadata:
    def __init__(
        self,
        metadata_file=None,
        input_folder=None,
        output_folder=None,
        mapping_illumina=None,
    ):
        if metadata_file is None:
            self.metadata_file = relecov_tools.utils.prompt_path(
                msg="Select the excel file which contains metadata"
            )
        else:
            self.metadata_file = metadata_file
        if not os.path.exists(self.metadata_file):
            log.error("Metadata file %s does not exist ", self.metadata_file)
            stderr.print(
                "[red] Metadata file " + self.metadata_file + " does not exist"
            )
            sys.exit(1)
        if input_folder is None:
            self.input_folder = relecov_tools.utils.prompt_path(
                msg="Select the input folder"
            )
        else:
            self.input_folder = input_folder
        if output_folder is None:
            self.output_folder = relecov_tools.utils.prompt_path(
                msg="Select the output folder"
            )
        else:
            self.output_folder = output_folder
        if mapping_illumina is None:
            self.mapping_illumina = relecov_tools.utils.prompt_path(
                msg="Select the mapping illumina file"
            )
        else:
            self.mapping_illumina = mapping_illumina

        config_json = ConfigJson()
        relecov_schema = config_json.get_topic_data("json_schemas", "relecov_schema")
        relecov_sch_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "schema", relecov_schema
        )
        self.configuration = config_json

        with open(relecov_sch_path, "r") as fh:
            self.relecov_sch_json = json.load(fh)
        self.schema_name = self.relecov_sch_json["schema"]
        self.schema_version = self.relecov_sch_json["version"]

    def bioinfo_parse(self, file_name):
        """Fetch the metadata file folder  Directory to fetch metadata file
        file_name   metadata file name
        """
        # FUNCTION read_files
        """
        Read all the files and create dataframes
        """

        wb_file = openpyxl.load_workbook(file_name, data_only=True)
        ws_metadata_lab = wb_file["METADATA_LAB"]
        config_json = ConfigJson()
        relecov_bioinfo_metadata = config_json.get_configuration(
            "relecov_bioinfo_metadata"
        )
        c = 0
        self.files_read_bioinfo_metadata = config_json.get_configuration(
            "files_read_bioinfo_metadata"
        )

        mapping_illumina_tab_path = os.path.join(
            self.input_folder, self.mapping_illumina
        )

        summary_variants_metrics_path = os.path.join(
            self.input_folder, "summary_variants_metrics_mqc.csv"
        )
        variants_long_table_path = os.path.join(
            self.input_folder, "variants_long_table.csv"
        )
        consensus_genome_length_path = os.path.join(
            self.input_folder, "consensus_genome_length.csv"
        )
        software_versions_path = os.path.join(
            self.input_folder, "software_versions.yml"
        )
        pangolin_versions_path = os.path.join(self.input_folder, "pangolin_version.csv")
        self.md5_file_name = config_json.get_configuration("md5_file_name")
        md5_info_path = os.path.join(
            self.input_folder,
            self.md5_file_name,  # como hacer esto general para los servicios
        )

        mapping_illumina_tab = pd.read_csv(
            mapping_illumina_tab_path, sep=",", encoding="utf-8"
        )
        summary_variants_metrics = pd.read_csv(
            summary_variants_metrics_path, sep=",", encoding="utf-8"
        )
        variants_long_table = pd.read_csv(
            variants_long_table_path,
            sep=",",
            encoding="utf-8",
            dtype={"SAMPLE": "string"},
        )
        consensus_genome_length = pd.read_csv(
            consensus_genome_length_path, header=None, sep="\t", encoding="utf-8"
        )
        md5_info = pd.read_csv(md5_info_path, header=None, sep=",", encoding="utf-8")
        pangolin_version_table = pd.read_csv(
            pangolin_versions_path, header=None, sep="\t", encoding="utf-8"
        )

        with open(software_versions_path) as file:
            software_versions = yaml.load(file, Loader=yaml.FullLoader)

        self.mapping_illumina_tab_field_list = config_json.get_configuration(
            "mapping_illumina_tab_field_list"
        )
        # FUNCTION fill_bioinfo_dict
        """Iterating through each row and each loaded file the values of the bioinfo_dict are filled"""

        bioinfo_list = []

        for row in islice(ws_metadata_lab.values, 4, ws_metadata_lab.max_row):
            if row[6] is None:
                continue
            sample_name = row[6]
            print(sample_name)

            fastq_r1 = row[47]

            fastq_r2 = row[48]
            bioinfo_dict = {}
            bioinfo_dict["sample_name"] = str(sample_name)
            bioinfo_dict["sequence_file_R1_fastq"] = fastq_r1
            bioinfo_dict["sequence_file_R2_fastq"] = fastq_r2
            # FUNCTION config_data
            """inserting all keys from configuration.json  relecov_bioinfo_metadata into bioinfo_dict"""
            for key in relecov_bioinfo_metadata.keys():
                bioinfo_dict[key] = relecov_bioinfo_metadata[key]
            bioinfo_dict["consensus_sequence_filepath"] = self.input_folder
            bioinfo_dict["long_table_path"] = self.input_folder
            # FUNCTION mapping_illumina_data
            # fields from mapping_illumina.tab
            mapping_illumina_tab_sample = mapping_illumina_tab[
                mapping_illumina_tab["sample"].str.contains(bioinfo_dict["sample_name"])
            ]
            config_keys = list(self.mapping_illumina_tab_field_list.keys())
            for i in config_keys:
                bioinfo_dict[i] = str(
                    mapping_illumina_tab_sample[
                        self.mapping_illumina_tab_field_list[i]
                    ].values[0]
                )
            # FUNCTION summary_metrics_data
            # fields from summary_variants_metrics_mqc.csv
            bioinfo_dict["number_of_base_pairs_sequenced"] = str(
                summary_variants_metrics.loc[
                    summary_variants_metrics["Sample"].str.contains(
                        bioinfo_dict["sample_name"]
                    )
                ]["# Input reads"].values[0]
                * 2
            )

            bioinfo_dict["ns_per_100_kbp"] = str(
                summary_variants_metrics.loc[
                    summary_variants_metrics["Sample"].str.contains(
                        bioinfo_dict["sample_name"]
                    )
                ]["# Ns per 100kb consensus"].values[0]
            )

            bioinfo_dict["qc_filtered"] = str(
                summary_variants_metrics.loc[
                    summary_variants_metrics["Sample"].str.contains(
                        bioinfo_dict["sample_name"]
                    )
                ]["# Trimmed reads (fastp)"].values[0]
            )
            # FUNCTION long_table_data
            # fields from variants_long_table.csv

            if os.path.isfile(
                self.input_folder + bioinfo_dict["sample_name"] + ".consensus.fa"
            ):
                chrom = variants_long_table.loc[
                    variants_long_table["SAMPLE"].str.contains(
                        bioinfo_dict["sample_name"]
                    )
                ]["CHROM"]
                bioinfo_dict["reference_genome_accession"] = str(chrom.values[0])
            else:
                bioinfo_dict["reference_genome_accession"] = "NC_045512.2"
            # FUNCTION genome_length_data
            # fields from consensus_genome_length
            cons_array = consensus_genome_length.loc[
                consensus_genome_length[0].str.contains(bioinfo_dict["sample_name"])
            ]
            if len(cons_array) > 1:
                for i in cons_array:
                    if cons_array.values[i, 1] != 0:
                        bioinfo_dict["consensus_genome_length"] = str(
                            cons_array.values[i, 1]
                        )
            # FUNCTION md5_data
            # fields from md5 file

            for i in range(len(md5_info)):
                if "-2" in bioinfo_dict["sequence_file_R1_fastq"]:
                    g = re.match(
                        str(bioinfo_dict["sequence_file_R1_fastq"][0:8]) + "$",
                        md5_info[0][i],
                    )
                elif "-B" in bioinfo_dict["sequence_file_R1_fastq"]:

                    g = re.match(
                        str(bioinfo_dict["sequence_file_R1_fastq"][0:8]) + "$",
                        md5_info[0][i],
                    )
                else:
                    g = re.match(str(bioinfo_dict["sample_name"]) + "$", md5_info[0][i])
                if g is not None:
                    bioinfo_dict["consensus_sequence_R1_name"] = md5_info[2][i]
                    bioinfo_dict["consensus_sequence_R2_name"] = md5_info[2][i + 1]
                    bioinfo_dict["consensus_sequence_R1_md5"] = md5_info[1][i]
                    bioinfo_dict["consensus_sequence_R2_md5"] = md5_info[1][i + 1]
                    break
            # FUNCTION software_data
            # fields from software version file
            bioinfo_dict["dehosting_method_software_version"] = str(
                software_versions["KRAKEN2_KRAKEN2"]["kraken2"]
            )
            bioinfo_dict["variant_calling_software_version"] = str(
                software_versions["IVAR_VARIANTS"]["ivar"]
            )
            bioinfo_dict["consensus_sequence_software_version"] = str(
                software_versions["BCFTOOLS_CONSENSUS"]["bcftools"]
            )

            bioinfo_dict["bioinformatics_protocol_software_version"] = str(
                software_versions["Workflow"]["nf-core/viralrecon"]
            )

            bioinfo_dict["preprocessing_software_version"] = str(
                software_versions["FASTP"]["fastp"]
            )
            bioinfo_dict["mapping_software_version"] = str(
                software_versions["BOWTIE2_ALIGN"]["bowtie2"]
            )

            # FUNCTION pangolin_data
            # files from pangolin.csv file

            bioinfo_dict["lineage_analysis_software_version"] = str(
                pangolin_version_table.loc[
                    pangolin_version_table[0].str.contains(bioinfo_dict["sample_name"])
                ].values[0][1]
            )
            bioinfo_dict["variant_designation"] = str(
                pangolin_version_table.loc[
                    pangolin_version_table[0].str.contains(bioinfo_dict["sample_name"])
                ].values[0][2]
            )

            # get the date from pangolin files
            string = re.split("(\d+)", self.mapping_illumina)[1]
            year, month, day = int(string[:4]), int(string[4:6]), int(string[6:-1])
            x = datetime.datetime(year, month, day)
            bioinfo_dict["analysis_date"] = x.strftime("%b %d %Y %H:%M:%S")

            pango_file_path = (
                self.input_folder + bioinfo_dict["sample_name"] + ".pangolin.csv"
            )
            if os.path.isfile(pango_file_path):
                pango_last_modified = os.path.getctime(pango_file_path)
                pango_last_modified_date = datetime.datetime.fromtimestamp(
                    pango_last_modified
                )
                bioinfo_dict["lineage_identification_date"] = str(
                    pango_last_modified_date
                )
            bioinfo_list.append(bioinfo_dict)
            # bioinfo_list[str(sample_name)] = bioinfo_dict
            c = c + 1
            # adding schema_name and schema_version
            bioinfo_dict["schema_name"] = self.schema_name
            bioinfo_dict["schema_version"] = self.schema_version

        json_file = "bioinfo_metadata.json"
        output_path = os.path.join(self.output_folder, json_file)

        with open(output_path, "w", encoding="utf-8") as fh:
            fh.write(
                json.dumps(bioinfo_list, indent=4, sort_keys=True, ensure_ascii=False)
            )



#!/usr/bin/env python
import sys

import logging
import rich.console
import relecov_tools.utils

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


def replace_originating_lab(metadata, f_data, mapped_fields, heading):
    """Replace the format text in the originating lab and replace by the ones
    defined in the System
    """
    for row in metadata[1:]:
        for key, val in mapped_fields.items():
            meta_idx = heading.index(key)
            try:
                row[meta_idx] = f_data[row[meta_idx].strip()][val].strip()
            except KeyError as e:
                log.error("Value  %s does not exist ", e)
                stderr.print(f"[red] Value {e} does not exist")
                sys.exit(1)
    return metadata


def added_seq_inst_model(metadata, f_data, mapped_fields, heading):
    """Set the type of sequencer instrument based on the run name"""
    s_idx = heading.index("Sample ID given for sequencing")
    for row in metadata[1:]:
        for key, val in mapped_fields.items():
            m_idx = heading.index(key)

            try:
                run_name = f_data[str(row[s_idx])][val].lower()
            except KeyError as e:
                log.error("Value  %s does not exist ", e)
                stderr.print(f"[red] Value {e} does not exist")
                sys.exit(1)
            if "nextseq" in run_name:
                row[m_idx] = "Illumina NextSeq 500"
            elif "next_seq" in run_name:
                row[m_idx] = "Illumina NextSeq 500"
            elif "miseq" in run_name:
                row[m_idx] = "Illumina MiSeq"
            elif "miseaq" in run_name:
                row[m_idx] = "Illumina MiSeq"
            elif "novaseq" in run_name:
                row[m_idx] = "Illumina NovaSeq 6000"
            else:
                log.error("Value  %s is not defined in the mapping ", run_name)
                stderr.print(f"[red] Value {run_name} is not defined in the mapping")
                sys.exit(1)
    return metadata


def translate_gender_to_english(metadata, f_data, mapped_fields, heading):
    """Translate into english the host gender that is written in spanish"""
    for row in metadata[1:]:
        for key, val in mapped_fields.items():
            m_idx = heading.index(key)
            if row[m_idx] is None:
                row[m_idx] = "not provided"
            elif "hombre" in row[m_idx].lower():
                row[m_idx] = "Male"
            elif "mujer" in row[m_idx].lower():
                row[m_idx] = "Female"
            elif "desconocido" in row[m_idx].lower():
                row[m_idx] = "not provided"
            elif "Unknown" in row[m_idx].lower():
                row[m_idx] = "not provided"
            elif "unknown" in row[m_idx].lower():
                row[m_idx] = "not provided"
            else:
                row[m_idx] = "not applicable"
    return metadata



import logging

# from re import template

# from pyparsing import col
import xml.etree.ElementTree as ET
import rich.console
import json


import pandas as pd


import sys
import os

# import ftplib
import relecov_tools.utils
from relecov_tools.config_json import ConfigJson

from ena_upload.ena_upload import extract_targets

# from ena_upload.ena_upload import submit_data
from ena_upload.ena_upload import run_construct
from ena_upload.ena_upload import construct_submission
from ena_upload.ena_upload import send_schemas
from ena_upload.ena_upload import process_receipt
from ena_upload.ena_upload import update_table

# from ena_upload.ena_upload import make_update
# from ena_upload.ena_upload import process_receipt

# from ena_upload.ena_upload import save_update
import site

pd.options.mode.chained_assignment = None

template_path = os.path.join(site.getsitepackages()[0], "ena_upload", "templates")
template_path = os.path.join(os.getcwd(), "relecov_tools/templates")

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class EnaUpload:
    def __init__(
        self,
        user=None,
        passwd=None,
        center=None,
        source_json=None,
        dev=None,
        customized_project=None,
        action=None,
        output_path=None,
    ):
        if user is None:
            self.user = relecov_tools.utils.prompt_text(
                msg="Enter your username defined in ENA"
            )
        else:
            self.user = user
        if passwd is None:
            self.passwd = relecov_tools.utils.prompt_password(
                msg="Enter your password to ENA"
            )
        else:
            self.passwd = passwd
        if center is None:
            self.center = relecov_tools.utils.prompt_text(msg="Enter your center name")
        else:
            self.center = center
        if source_json is None:
            self.source_json_file = relecov_tools.utils.prompt_path(
                msg="Select the ENA json file to upload"
            )
        else:
            self.source_json_file = source_json
        if dev is None:
            self.dev = relecov_tools.utils.prompt_yn_question(
                msg="Do you want to test upload data?"
            )
        else:
            self.dev = dev

        if customized_project is None:
            self.customized_project = None
        else:
            self.customized_project = customized_project
        if action is None:
            self.action = relecov_tools.utils.prompt_selection(
                msg="Select the action to upload to ENA",
                choices=["add", "modify", "cancel", "release"],
            )
        else:
            self.action = action.upper()
        if output_path is None:
            self.output_path = relecov_tools.utils.prompt_path(
                msg="Select the folder to store the xml files"
            )
        else:
            self.output_path = output_path

        if not os.path.isfile(self.source_json_file):
            log.error("json data file %s does not exist ", self.source_json_file)
            stderr.print(f"[red]json data file {self.source_json_file} does not exist")
            sys.exit(1)
        with open(self.source_json_file, "r") as fh:
            self.json_data = json.loads(fh.read())

    def convert_input_json_to_ena(self):
        """Split the input ena json, in samples and runs json"""
        pass

    def create_structure_to_ena(self):
        """Convert json to dataframe required by ena-upload-cli package"""

        squema = self.source_json_file
        fh_squema = open(squema)
        squema_json = json.load(fh_squema)
        fh_squema.close()

        df_schemas = pd.DataFrame(squema_json)

        # df_schema
        """
        collecting_institution collection_date     collector_name  ...                                        study_title    study_type taxon_id
0      Hospital Clínic de Barcelona      2021-05-10  Inmaculada Casas   ...  RELECOV Spanish Network for genomics surveillance  Surveillance  2697049
1      Hospital Clínic de Barcelona      2021-05-07   Inmaculada Casas  ...  RELECOV Spanish Network for genomics surveillance  Surveillance  2697049

        """
        """
        df_study = df_schemas[
            ["study_alias", "study_title", "study_type", "study_abstract"]
        ]

        df_study = df_study.rename(columns={"study_alias": "alias"})
        df_study = df_study.rename(columns={"study_title": "title"})
        """
        df_study = df_schemas[["study_type", "study_abstract"]]
        df_study["alias"] = df_study["study_abstract"][0].rsplit(" ", 5)[0]
        df_study["title"] = df_study["study_abstract"]
        df_study.insert(3, "status", self.action)
        """
        # df_study
        alias                                              title    study_type status                                     study_abstract
0   RELECOV  RELECOV Spanish Network for genomics surveillance  Surveillance    ADD  RELECOV Spanish Network for genomics surveillance
1   RELECOV  RELECOV Spanish Network for genomics surveillance  Surveillance    ADD  RELECOV Spanish Network for genomics surveillance
        """
        df_samples = df_schemas[
            [
                "sample_name",
                "sample_title",
                "taxon_id",
                "collection_date",
                "geographic_location_(country_and/or_sea)",
                "host_common_name",
                "host_scientific_name",
                "host_sex",
                "scientific_name",
                "collector_name",
                "collecting_institution",
                "isolate",
                "host_subject_id",
                "host health state",
                "authors",
                "sample_description",
                "address",
            ]
        ]

        df_samples["host_sex"].replace("unknown", "not provided", inplace=True)
        df_samples["host_sex"].replace("Unknown", "not provided", inplace=True)

        df_samples = df_samples.rename(columns={"sample_name": "alias"})
        df_samples = df_samples.rename(columns={"sample_title": "title"})
        df_samples = df_samples.rename(
            columns={
                "geographic_location_(country_and/or_sea)": "geographic location (country and/or sea)"
            }
        )
        df_samples = df_samples.rename(columns={"host_subject_id": "host subject id"})
        df_samples = df_samples.rename(columns={"collection_date": "collection date"})
        df_samples = df_samples.rename(columns={"host_common_name": "host common name"})
        df_samples = df_samples.rename(columns={"host_common_name": "host common name"})
        df_samples = df_samples.rename(columns={"host_sex": "host sex"})
        df_samples = df_samples.rename(
            columns={"host_scientific_name": "host scientific name"}
        )
        df_samples = df_samples.rename(columns={"collector_name": "collector name"})
        df_samples = df_samples.rename(
            columns={"collecting_institution": "collecting institution"}
        )
        df_samples.insert(3, "status", self.action)
        # df_samples.insert(3, "host subject id", df_schemas["host subject id"])
        # df_samples.insert(3, "host health state", df_schemas["host health state"])
        config_json = ConfigJson()
        checklist = config_json.get_configuration("checklist")
        df_samples.insert(4, "ENA_CHECKLIST", checklist)
        # df_samples.insert(5, "sample_description", df_schemas["sample_description"])
        # df_samples

        """
        alias      title taxon_id host health state  ...                                  scientific_name     collector name           collecting institution    isolate
0   212164375  212164375  2697049                    ...  Severe acute respiratory syndrome coronavirus 2  Inmaculada Casas      Hospital Clínic de Barcelona  212164375
1   212163777  212163777  2697049                    ...  Severe acute respiratory syndrome coronavirus 2   Inmaculada Casas     Hospital Clínic de Barcelona  212163777
2   212153091  212153091  2697049                    ...  Severe acute respiratory syndrome coronavirus 2   Inmaculada Casas     Hospital Clínic de Barcelona  212153091
        """
        import pdb

        pdb.set_trace()
        df_run = df_schemas[
            [
                "experiment_alias",
                "r1_fastq_filepath",
                "r2_fastq_filepath",
                "file_type",
                "fastq_r1_md5",
                "fastq_r2_md5",
                "collecting_institution",
            ]
        ]

        df_run.insert(1, "sequence_file_R1_fastq", "None")
        df_run.insert(2, "sequence_file_R2_fastq", "None")
        for i in range(len(df_schemas)):
            df_run.loc[i, "sequence_file_R1_fastq"] = df_schemas.loc[
                i, "sequence_file_R1_fastq"
            ]

            df_run.loc[i, "sequence_file_R2_fastq"] = df_schemas.loc[
                i, "sequence_file_R2_fastq"
            ]
        df_run.insert(3, "status", self.action)
        df_run = df_run.rename(columns={"fastq_r1_md5": "file_checksum"})

        for i in range(len(df_run)):
            df_run.loc[i, "alias"] = (
                str(df_run.loc[i, "sequence_file_R1_fastq"])
                + "_"
                + str(df_run.loc[i, "sequence_file_R2_fastq"])
            )
            df_run.loc[i, "experiment_alias"] = (
                str(df_run.loc[i, "sequence_file_R1_fastq"])
                + "_"
                + str(df_run.loc[i, "sequence_file_R2_fastq"])
            )

        df_run.insert(5, "file_name", df_run["sequence_file_R1_fastq"])
        df_run2 = df_run.copy()
        df_run2["file_name"] = df_run["sequence_file_R2_fastq"]
        df_run2["file_checksum"] = df_run["fastq_r2_md5"]
        df_run_final = pd.concat([df_run, df_run2])
        df_run_final.reset_index()

        # df_run
        """
        experiment_alias      sequence_file_R1_fastq      sequence_file_R2_fastq  ... file_type                     file_checksum                      fastq_r2_md5
0        214821_S12  214821_S12_R1_001.fastq.gz  214821_S12_R2_001.fastq.gz  ...     fastq  372ca8b10a8eeb7a04107634baf340ab  7f5081eec1b64b171402b66f37fe640d
1        214821_S12  214822_S13_R1_001.fastq.gz  214822_S13_R2_001.fastq.gz  ...     fastq  b268d7be80e80455bec0807b5961d23c  a15be39feaf73eec9b2c026717878bba
2        214821_S12   214823_S1_R1_001.fastq.gz   214823_S1_R2_001.fastq.gz  ...     fastq  c16bdbfc03c354496fcfb2c107e3cbf6  4d9b80b977a75bf7e2a4282ca910d94a

        """

        df_experiments = df_schemas[
            [
                "experiment_alias",
                "study_title",
                "study_alias",
                "sample_name",
                "library_name",
                "library_strategy",
                "library_source",
                "library_selection",
                "library_layout",
                "instrument_model",
                "design_description",
                "collecting_institution",
                "insert_size",
                "sequencing_instrument_platform",
            ]
        ]

        df_experiments["instrument_model"] = df_experiments[
            "instrument_model"
        ].str.lower()

        df_experiments.insert(3, "status", self.action)

        for i in range(len(df_experiments)):
            df_experiments.loc[i, "alias"] = (
                str(df_run.loc[i, "sequence_file_R1_fastq"])
                + "_"
                + str(df_run.loc[i, "sequence_file_R2_fastq"])
            )
        # df_experiments.insert(5, "design_description", df_schemas["design_description"])
        # df_experiments.insert(5, "insert_size", df_schemas["insert_size"])
        # df_experiments.insert(5, "platform", df_schemas["sequencing_instrument_platform"])

        df_experiments = df_experiments.rename(
            columns={"sequencing_instrument_platform": "platform"}
        )
        df_experiments = df_experiments.rename(columns={"study_title": "title"})
        df_experiments = df_experiments.rename(columns={"sample_name": "sample_alias"})
        df_experiments = df_experiments.rename(
            columns={"collecting_institution": "collecting institution"}
        )

        # df_experiments example
        """
        experiment_alias                                              title study_alias  ... library_layout instrument_model                                              alias
0        214821_S12  RELECOV Spanish Network for genomics surveillance     RELECOV  ...         PAIRED   Illumina MiSeq  214821_S12_R1_001.fastq.gz_214821_S12_R2_001.f...
1        214821_S12  RELECOV Spanish Network for genomics surveillance     RELECOV  ...         PAIRED   Illumina MiSeq  214822_S13_R1_001.fastq.gz_214822_S13_R2_001.f...
2        214821_S12  RELECOV Spanish Network for genomics surveillance     RELECOV  ...         PAIRED   Illumina MiSeq  214823_S1_R1_001.fastq.gz_214823_S1_R2_001.fas...

        """

        ena_config = config_json.get_configuration("ENA_configuration")
        schema_dataframe = {}
        schema_dataframe["sample"] = df_samples
        schema_dataframe["run"] = df_run_final
        schema_dataframe["experiment"] = df_experiments
        schema_targets = extract_targets(self.action, schema_dataframe)

        if ena_config["study_id"] is not None:
            schema_dataframe["study"] = df_study

        if (
            self.action == "ADD"
            or self.action == "add"
            or self.action == "MODIFY"
            or self.action == "modify"
        ):
            file_paths = {}
            file_paths_r2 = {}

            for path in df_schemas["r1_fastq_filepath"]:
                file_paths[os.path.basename(path)] = os.path.abspath(path)

            for path in df_schemas["r2_fastq_filepath"]:
                file_paths_r2[os.path.basename(path)] = os.path.abspath(path)

            file_paths.update(file_paths_r2)

            # submit data to webin ftp server. It should only upload fastq files in case the action is ADD.
            # # When the action is MODIFY rthe fastq are already submitted
            """
            if self.action == "ADD" or self.action == "add":
                session = ftplib.FTP("webin2.ebi.ac.uk", self.user, self.passwd)

                for filename, path in file_paths.items():

                    print("Uploading path " + path + " and filename: " + filename)

                    try:
                        file = open(path, "rb")  # file to send
                        g = session.storbinary(f"STOR {filename}", file)
                        print(g)  # send the file
                        file.close()  # close file and FTP
                    except BaseException as err:

                        print(f"ERROR: {err}")
                        # print("ERROR: If your connection times out at this stage, it propably is because of a firewall that is in place. FTP is used in passive mode and connection will be opened to one of the ports: 40000 and 50000.")

                g2 = session.quit()
                print(g2)
            """

            # THE ENA_UPLOAD_CLI METHOD DOES NOT WORK (below)
            # chec = submit_data(file_paths, self.passwd, self.user)
            # print(chec)

            # when ADD/MODIFY,
            #
            # requires source XMLs for 'run', 'experiment', 'sample', 'experiment'
            # schema_xmls record XMLs for all these schema and following 'submission'

            tool = config_json.get_configuration("tool")

            schema_xmls = run_construct(
                template_path, schema_targets, self.center, checklist, tool
            )

            # submission_xml = construct_submission(template_path, self.action, schema_xmls, self.center, checklist, tool)
            submission_xml = construct_submission(
                template_path, self.action, schema_xmls, self.center, checklist, tool
            )
            schema_xmls["submission"] = submission_xml

            tree = ET.parse(schema_xmls["run"])
            root = tree.getroot()

            for files in root.iter("FILE"):
                if "R2" in files.attrib["filename"]:
                    # print(files.attrib["filename"])
                    H = df_run_final.loc[
                        df_run_final["sequence_file_R2_fastq"]
                        == files.attrib["filename"]
                    ].values[0][9]
                    files.set("checksum", H)
                    # print(files.attrib["checksum"])
            tree.write(schema_xmls["run"])

            if self.dev:
                url = "https://wwwdev.ebi.ac.uk/ena/submit/drop-box/submit/?auth=ENA"
            else:
                url = "https://www.ebi.ac.uk/ena/submit/drop-box/submit/?auth=ENA"

            print(f"\nSubmitting XMLs to ENA server: {url}")

            receipt = send_schemas(schema_xmls, url, self.user, self.passwd).text
            if not os.path.exists(self.output_path):
                os.mkdir(self.output_path)
            receipt_dir = os.path.join(self.output_path, "receipt.xml")
            print(f"Printing receipt to {receipt_dir}")

            with open(f"{receipt_dir}", "w") as fw:
                fw.write(receipt)
            try:
                schema_update = process_receipt(receipt.encode("utf-8"), self.action)
            except ValueError:
                log.error("There was an ERROR during submission:")
                sys.exit(receipt)

            if self.action in ["ADD", "MODIFY"] or self.action in ["add", "modify"]:
                schema_dataframe = update_table(
                    schema_dataframe, schema_targets, schema_update
                )
            ######

    def upload(self):
        """Create the required files and upload to ENA"""
        self.convert_input_json_to_ena()
        self.create_structure_to_ena()



#!/usr/bin/env python
"""
Common utility function for relecov_tools package.
"""
import os
import glob
import hashlib
import logging
from rich.console import Console
import questionary
import json
import openpyxl
from itertools import islice

log = logging.getLogger(__name__)


def file_exists(file_to_check):
    """
    Input:
        file_to_check   # file name to check if exists
    Return:
        True if exists
    """
    if os.path.isfile(file_to_check):
        return True
    return False


def read_json_file(j_file):
    """Read json file."""
    with open(j_file, "r") as fh:
        data = json.load(fh)
    return data


def read_execl_file(f_name, sheet_name):
    """Read the input excel file and give the information in a list
    of dictionaries
    """
    wb_file = openpyxl.load_workbook(f_name, data_only=True)
    ws_metadata_lab = wb_file[sheet_name]
    heading = [i.value.strip() for i in ws_metadata_lab[1] if i.value]
    ws_data = []
    for row in islice(ws_metadata_lab.values, 1, ws_metadata_lab.max_row):
        l_row = list(row)
        data_row = {}
        # Ignore the empty rows
        # guessing that row 1 and 2 with no data are empty rows
        if l_row[0] is None and l_row[1] is None:
            continue
        for idx in range(0, len(heading)):
            data_row[heading[idx]] = l_row[idx]
        ws_data.append(data_row)

    return ws_data


def read_csv_file_return_dict(file_name, sep):
    """Read csv or tsv file, according to separator, and return a dictionary
    where the main key is the first column
    """
    with open(file_name, "r") as fh:
        lines = fh.readlines()
    heading = lines[0].strip().split(sep)
    if len(heading) == 0:
        return {"ERROR": "not valid format"}
    file_data = {}
    for line in lines[1:]:
        line_s = line.strip().split(sep)
        file_data[line_s[0]] = {}
        for idx in range(1, len(heading)):
            file_data[line_s[0]][heading[idx]] = line_s[idx]
    return file_data


def get_md5_from_local_folder(local_folder):
    """Fetch the md5 values for each file in the file list"""
    md5_results = {}
    reg_for_md5 = os.path.join(local_folder, "*.md5")
    # reg_for_non_md5 = os.path.join(local_folder, "*[!.md5]")
    md5_files = glob.glob(reg_for_md5)
    if len(md5_files) > 0:
        for md5_file in md5_files:
            file_path_name, f_ext = os.path.splitext(md5_file)
            if not file_exists(file_path_name):
                log.error("Found md5 file but not %s", file_path_name)
                continue
            file_name = os.path.basename(file_path_name)
            fh = open(md5_file, "r")
            md5_results[file_name] = [local_folder, fh.read()]
            fh.close()
    return md5_results


def calculate_md5(file_name):
    """Calculate the md5 value for the file name"""
    return hashlib.md5(open(file_name, "rb").read()).hexdigest()


def write_md5_file(file_name, md5_value):
    """Write md5 to file"""
    with open(file_name, "w") as fh:
        fh.write(md5_value)
    return


def create_md5_files(local_folder, file_list):
    """Create the md5 files and return their value"""
    md5_results = {}
    for file_name in file_list:
        md5_results[file_name] = [
            local_folder,
            calculate_md5(os.path.join(local_folder, file_name)),
        ]
        md5_file_name = file_name + ".md5"
        write_md5_file(
            os.path.join(local_folder, md5_file_name), md5_results[file_name][1]
        )
    return md5_results


def save_local_md5(file_name, md5_value):
    """Save the MD5 value"""
    with open(file_name, "w") as fh:
        fh.write(md5_value)
    return True


def write_to_excel_file(data, f_name, sheet_name, post_process=None):
    book = openpyxl.Workbook()
    sheet = book.active

    for row in data:
        sheet.append(row)
    # adding one column with row number
    if "insert_cols" in post_process:
        sheet.insert_cols(post_process["insert_cols"])
        sheet["A1"] = "Campo"
        counter = 1
        for i in range(len(data)):
            idx = "A" + str(counter + 1)
            sheet[idx] = counter
            counter += 1
    # adding 3 empty rows
    if "insert_rows" in post_process:
        for x in range(post_process["insert_rows"]):
            sheet.insert_rows(1)
        sheet.title = sheet_name
    book.save(f_name)
    return


def rich_force_colors():
    """
    Check if any environment variables are set to force Rich to use coloured output
    """
    if (
        os.getenv("GITHUB_ACTIONS")
        or os.getenv("FORCE_COLOR")
        or os.getenv("PY_COLORS")
    ):
        return True
    return None


stderr = Console(
    stderr=True, style="dim", highlight=False, force_terminal=rich_force_colors()
)


def prompt_text(msg):
    source = questionary.text(msg).unsafe_ask()
    return source


def prompt_password(msg):
    source = questionary.password(msg).unsafe_ask()
    return source


def prompt_tmp_dir_path():
    stderr.print("Temporal directory destination to execute service")
    source = questionary.path("Source path").unsafe_ask()
    return source


def prompt_selection(msg, choices):
    selection = questionary.select(msg, choices=choices).unsafe_ask()
    return selection


def prompt_path(msg):
    source = questionary.path(msg).unsafe_ask()
    return source


def prompt_yn_question(msg):
    confirmation = questionary.confirm(msg).unsafe_ask()
    return confirmation


def prompt_skip_folder_creation():
    stderr.print("Do you want to skip folder creation? (Y/N)")
    confirmation = questionary.confirm("Skip?", default=False).unsafe_ask()
    return confirmation



#!/usr/bin/env python
import logging
import rich.console
import jsonschema
from jsonschema import validate
from jsonschema import Draft202012Validator
import json
import sys
import os
import openpyxl

import relecov_tools.utils
from relecov_tools.config_json import ConfigJson

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


def validate_json(json_data_file=None, json_schema_file=None, out_folder=None):
    """Validate json file against the schema"""
    if json_data_file is None:
        json_data_file = relecov_tools.utils.prompt_path(
            msg="Select the json file to validate"
        )
    if json_schema_file is None:
        config_json = ConfigJson()
        schema_name = config_json.get_topic_data("json_schemas", "relecov_schema")
        json_schema_file = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "schema", schema_name
        )
    # schema_file = open('/home/lchapado/Projects/Proyecto_ERA/relecov-tools/schema/phage_plus_V0.json')
    with open(json_schema_file, "r") as fh:
        json_schema = json.load(fh)
    try:
        Draft202012Validator.check_schema(json_schema)
    except jsonschema.ValidationError:
        stderr.print("[red] Json schema does not fulfil Draft 202012 Validation")
        sys.exit(1)
    if not os.path.isfile(json_data_file):
        stderr.print("[red] Json file does not exists")
        sys.exit(1)
    stderr.print("[blue] Reading the json file")
    json_data = relecov_tools.utils.read_json_file(json_data_file)
    with open(json_data_file, "r") as fh:
        json_data = json.load(fh)
    validated_json_data = []
    invalid_json = []
    errors = {}
    stderr.print("[blue] Start processing the json file")
    for item_row in json_data:
        try:
            validate(instance=item_row, schema=json_schema)
            validated_json_data.append(item_row)
        except jsonschema.ValidationError as e:
            log.error("Invalid sample data %s", e)
            invalid_json.append(item_row)
    # Enviar los errores por correo
    # logging.handlers.SMTPHandler(mailhost=("smtp.gmail.com", 465), fromaddr=correo_isciii, toaddrs=correo_usuario, subject="Validation errors", credentials=(usurario,contraseña), secure=None, timeout=1.0)
    if len(invalid_json) == 0:
        stderr.print("[green] Sucessful validation")
    else:
        stderr.print("[red] Some samples are not validated")
    return validated_json_data, invalid_json, errors


def create_invalid_metadata(metadata_file, invalid_json, out_folder):
    """Create a new sub excel file having only the samples that were invalid.
    Samples name are checking the Sequencing sample id which are in
    column B (index 1).
    The rows that match the value collected from json file on tag
    collecting_lab_sample_id are removed from excel
    """
    sample_list = []
    # import pdb; pdb.set_trace()
    # json_data = relecov_tools.utils.read_json_file(invalid_json)
    stderr.print("[red] Start preparation of invalid samples")
    for row in invalid_json:
        sample_list.append(str(row["collecting_lab_sample_id"]))
    wb = openpyxl.load_workbook(metadata_file)
    ws_sheet = wb["METADATA_LAB"]
    row_to_del = []

    for row in ws_sheet.iter_rows(min_row=5, max_row=ws_sheet.max_row):
        # if not data on row 1 and 2 assume that no more data are in file
        # then start deleting rows
        if not row[2].value and not row[1].value:
            if len(row_to_del) > 0:
                row_to_del.sort(reverse=True)
                for idx in row_to_del:
                    ws_sheet.delete_rows(idx)
            break

        if str(row[2].value) not in sample_list:
            row_to_del.append(row[0].row)

    os.makedirs(out_folder, exist_ok=True)
    new_name = "invalid_" + os.path.basename(metadata_file)
    m_file = os.path.join(out_folder, new_name)
    stderr.print("[red] Saving excel file with the invalid samples")
    wb.save(m_file)
    return



#!/usr/bin/env python
from itertools import islice

import json
import logging

# from queue import Empty

# from turtle import heading
import rich.console

import openpyxl
import os
import sys
import re
import relecov_tools.utils
from relecov_tools.config_json import ConfigJson
import relecov_tools.json_schema

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=relecov_tools.utils.rich_force_colors(),
)


class RelecovMetadata:
    def __init__(self, metadata_file=None, sample_list_file=None, output_folder=None):
        if metadata_file is None:
            self.metadata_file = relecov_tools.utils.prompt_path(
                msg="Select the excel file which contains metadata"
            )
        else:
            self.metadata_file = metadata_file
        if not os.path.exists(self.metadata_file):
            log.error("Metadata file %s does not exist ", self.metadata_file)
            stderr.print("[red] Metadata file " + self.meta_file + " does not exist")
            sys.exit(1)
        if sample_list_file is None:
            self.sample_list_file = relecov_tools.utils.prompt_path(
                msg="Select the file which contains the sample information"
            )
        else:
            self.sample_list_file = sample_list_file
        if not os.path.exists(self.sample_list_file):
            log.error("Sample information file %s does not exist ", self.metadata_file)
            stderr.print(
                "[red] Sample information " + self.sample_list_file + " does not exist"
            )
            sys.exit(1)
        if output_folder is None:
            self.output_folder = relecov_tools.utils.prompt_path(
                msg="Select the output folder"
            )
        else:
            self.output_folder = output_folder
        config_json = ConfigJson()
        relecov_schema = config_json.get_topic_data("json_schemas", "relecov_schema")
        relecov_sch_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "schema", relecov_schema
        )
        self.configuration = config_json

        with open(relecov_sch_path, "r") as fh:
            self.relecov_sch_json = json.load(fh)
        self.label_prop_dict = {}
        for prop, values in self.relecov_sch_json["properties"].items():
            try:
                self.label_prop_dict[values["label"]] = prop
            except KeyError:
                continue
        self.schema_name = self.relecov_sch_json["schema"]
        self.schema_version = self.relecov_sch_json["version"]

    def fetch_metadata_file(folder, file_name):
        """Fetch the metadata file folder  Directory to fetch metadata file
        file_name   metadata file name
        """
        wb_file = openpyxl.load_workbook(file_name, data_only=True)
        ws_metadata_lab = wb_file["METADATA_LAB"]
        heading = []
        for cell in ws_metadata_lab[1]:
            heading.append(cell.value)

    def read_json_file(self, j_file):
        """Read json file."""
        with open(j_file, "r") as fh:
            data = json.load(fh)
        return data

    def get_laboratory_data(self, lab_json, geo_loc_json, lab_name):
        """Fetch the laboratory location  and return a dictionary"""
        data = {}
        if lab_name == "":
            data["geo_loc_city"] = ""
            data["geo_loc_latitude"] = ""
            data["geo_loc_longitude"] = ""
            data["geo_loc_country"] = ""
            stderr.print("[red] Empty Originating Laboratory.")
            log.error("Found empty Originating Laboratory")
            return data
        for lab in lab_json:
            if lab_name == lab["collecting_institution"]:
                for key, value in lab.items():
                    data[key] = value
                break

        for city in geo_loc_json:
            try:
                if city["geo_loc_city"] == lab["geo_loc_city"]:
                    data["geo_loc_latitude"] = city["geo_loc_latitude"]
                    data["geo_loc_longitude"] = city["geo_loc_longitude"]
                    data["geo_loc_country"] = data["geo_loc_country"]
                    break
            except KeyError as e:
                print(e)
        return data

    def include_fixed_data(self):
        """Include fixed data that are always the same for each samples"""
        fixed_data = {
            "host_disease": "COVID-19",
            "type": "betacoronavirus",
            "tax_id": "2697049",
            "organism": "Severe acute respiratory syndrome coronavirus 2",
            "common_name": "Severe acute respiratory syndrome",
            "sample_description": "Sample for surveillance",
        }
        fixed_data.update(self.configuration.get_configuration("ENA_configuration"))
        fixed_data["schema_name"] = self.schema_name
        fixed_data["schema_version"] = self.schema_version
        return fixed_data

    def include_fields_already_set(self, row_sample):
        processed_data = {}

        if row_sample["author_submitter"] == "":
            processed_data["collector_name"] = "unknown"
        else:
            processed_data["collector_name"] = row_sample["author_submitter"]
        processed_data["host_subject_id"] = row_sample["microbiology_lab_sample_id"]

        return processed_data

    def include_processed_data(self, metadata):
        """Include the data that requires to be processed to set the value"""
        new_data = {}
        p_data = {
            "host_common_name": {"Human": ["host_scientific_name", "Homo Sapiens"]},
            "collecting_lab_sample_id": [
                "isolate_sample_id",
                metadata["sequencing_sample_id"],
            ],
        }
        seq_inst_plat = {
            "Illumina": [
                "Illumina iSeq 100",
                "Illumina MiSeq",
                "Illumina NextSeq 550",
                "Illumina NextSeq",
                "Illumina NovaSeq 6000",
            ],
            "Oxford Nanopore": ["MinION"],
            "Ion Torrent": ["Ion Torrent S5", "Ion Torrent PGM"],
        }
        for key, values in p_data.items():
            v_data = metadata[key]
            if isinstance(values, dict):
                if v_data in values:
                    new_data[values[v_data][0]] = values[v_data][1]
            else:
                new_data[values[0]] = values[1]
        """New fields that required processing from other field """
        for key, values in seq_inst_plat.items():
            if metadata["sequencing_instrument_model"] in values:

                new_data["sequencing_instrument_platform"] = key
                break

        return new_data

    def add_additional_data(self, metadata, lab_json_file, geo_loc_file):
        """Add the additional information that must be included in final metadata
        metadata Origin metadata
        extra_data  additional data to be included
        result_metadata    final metadata after adding the additional data
        """
        lab_data = {}
        additional_metadata = []

        lab_json = self.read_json_file(lab_json_file)
        geo_loc_json = self.read_json_file(geo_loc_file)
        samples_json = self.read_json_file(self.sample_list_file)
        for row_sample in metadata:
            """Include sample data from sample json"""
            try:
                for key, value in samples_json[
                    row_sample["microbiology_lab_sample_id"]
                    # row_sample["sequencing_sample_id"]
                ].items():

                    row_sample[key] = value
            except KeyError as e:
                stderr.print(
                    "[red] ERROR  fastq information not found in sample json. ", e
                )

            """ Fetch the information related to the laboratory.
                Info is stored in lab_data, to prevent to call get_laboratory_data
                each time for each sample that belongs to the same lab
            """
            if row_sample["collecting_institution"] not in lab_data:
                # from collecting_institution find city, and geo location latitude and longitude
                l_data = self.get_laboratory_data(
                    lab_json, geo_loc_json, row_sample["collecting_institution"]
                )
                row_sample.update(l_data)
                lab_data[row_sample["collecting_institution"]] = l_data
            else:
                row_sample.update(lab_data[row_sample["collecting_institution"]])

            """ Fetch email and address for submitting_institution
            """
            row_sample["submitting_institution"] = row_sample[
                "submitting_institution"
            ].strip()
            if row_sample["submitting_institution"] not in lab_json:
                l_data = self.get_laboratory_data(
                    lab_json, geo_loc_json, row_sample["submitting_institution"]
                )
                # row_sample.update(l_data)
                lab_data[row_sample["submitting_institution"]] = l_data
            sub_data = {}

            sub_data["submitting_institution_email"] = lab_data[
                row_sample["submitting_institution"]
            ]["collecting_institution_email"]
            sub_data["submitting_institution_address"] = lab_data[
                row_sample["submitting_institution"]
            ]["collecting_institution_address"]
            # else:
            #    sub_data = {"collecting_institution_email" : "", "collecting_institution_address": ""}
            row_sample.update(sub_data)

            """ Add Fixed information
            """
            row_sample.update(self.include_fixed_data())

            """ Add fields that are already in other fields
            """
            row_sample.update(self.include_fields_already_set(row_sample))
            """Add information which requires processing
            """
            row_sample.update(self.include_processed_data(row_sample))
            """
            row["isolate"] = row["collecting_lab_sample_id"]
            row["host_scientific_name"] = extra_data["host_scientific_name"][
                row["host_common_name"]
            ]
            row["sequencing_instrument_platform"] = "To change"
            """
            # Add experiment_alias and run_alias
            row_sample["experiment_alias"] = str(
                row_sample["sequence_file_R1_fastq"]
                + "_"
                + row_sample["sequence_file_R2_fastq"]
            )
            row_sample["run_alias"] = str(
                row_sample["sequence_file_R1_fastq"]
                + "_"
                + row_sample["sequence_file_R2_fastq"]
            )
            additional_metadata.append(row_sample)

        return additional_metadata

    def request_information(external_url, request):
        """Get information from external database server using Rest API

        external_url
        request
        """
        pass

    def store_information(external_url, request, data):
        """Update information"""
        pass

    def read_metadata_file(self):
        """Read the input metadata file, changing the metadata heading with
        their property name values defined in schema.
        Convert the date colunms value to the dd/mm/yyyy format.
        Return list of dict with data, and errors
        """
        # exc_format_num = ["Sample ID given for sequencing"]
        wb_file = openpyxl.load_workbook(self.metadata_file, data_only=True)
        ws_metadata_lab = wb_file["METADATA_LAB"]

        # removing the None columns in excel heading row
        heading = [i.value.strip() for i in ws_metadata_lab[4] if i.value]

        # heading = self.update_heading_to_json(heading_without_none, meta_map_json)
        metadata_values = []
        errors = {}
        for row in islice(ws_metadata_lab.values, 4, ws_metadata_lab.max_row):
            sample_data_row = {}
            # Ignore the empty rows
            if row[2] is None:
                continue
            for idx in range(1, len(heading)):
                if "date" in heading[idx].lower():
                    try:
                        sample_data_row[self.label_prop_dict[heading[idx]]] = row[
                            idx
                        ].strftime("%Y-%m-%d")
                    except AttributeError:
                        # check if date is in string format
                        str_date = re.search(r"(\d{4}-\d{2}-\d{2}).*", row[idx])
                        if str_date:
                            sample_data_row[
                                self.label_prop_dict[heading[idx]]
                            ] = str_date.group(1)
                        else:
                            if row[2] not in errors:
                                errors[row[2]] = {}
                            errors[row[2]][heading[idx]] = "Invalid date format"
                            log.error("Invalid date format in sample %s", row[2])
                            stderr.print(
                                "[red] Invalid date format in sample",
                                row[2] + " column " + heading[idx],
                            )
                else:

                    if isinstance(row[idx], float) or isinstance(row[idx], int):
                        val = str(int(row[idx]))
                        try:
                            sample_data_row[self.label_prop_dict[heading[idx]]] = val
                        except TypeError as e:
                            stderr.print("[red] Error when reading " + row[2] + e)
                    else:
                        try:
                            sample_data_row[self.label_prop_dict[heading[idx]]] = (
                                row[idx] if row[idx] else ""
                            )
                        except KeyError as e:
                            stderr.print(
                                "[red] Error when reading " + str(row[2]) + str(e)
                            )
            metadata_values.append(sample_data_row)

        return metadata_values, errors

    def write_json_fo_file(self, data, file_name):
        """Write metadata to json file"""
        os.makedirs(self.output_folder, exist_ok=True)
        json_file = os.path.join(self.output_folder, file_name)
        with open(json_file, "w", encoding="utf-8") as fh:
            fh.write(json.dumps(data, indent=4, sort_keys=True, ensure_ascii=False))
        return True

    def create_metadata_json(self):
        stderr.print("[blue] Reading configuration settings")
        config_json = ConfigJson()
        geo_loc_json = config_json.get_configuration("geo_location_data")
        geo_loc_file = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "conf", geo_loc_json
        )
        lab_json = config_json.get_configuration("laboratory_data")
        lab_json_file = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "conf", lab_json
        )
        stderr.print("[blue] Reading Lab Metadata Excel File")
        valid_metadata_rows, errors = self.read_metadata_file()
        if len(errors) > 0:
            stderr.print("[red] Stopped executing because the errors found")
            sys.exit(1)
        # Continue by adding extra information
        stderr.print("[blue] Including additional information")
        completed_metadata = self.add_additional_data(
            valid_metadata_rows,
            lab_json_file,
            geo_loc_file,
        )

        file_name = (
            "processed_"
            + os.path.splitext(os.path.basename(self.metadata_file))[0]
            + ".json"
        )
        stderr.print("[blue] Creating Json file")
        self.write_json_fo_file(completed_metadata, file_name)
        return True
